//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to draw and label a colorbar
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class Colorbar {
    Colormap _colormap;     // The colormap 
    color    _colorsLow[];  // A color for each low vertex
    color    _colorsHigh[]; // A color for each high vertex
    float    _min, _max;    // Range of the colormap(s)
    float    _wide = 200;   // Width of the bar
    float    _high = 50;    // Height of the bar
    int      _numPts = 100; // The number of data pts to map
    float    _value;        // Current position in the map
    String   _label;        // Label
    Point    _origin;       // The origin of the colorbar

    // * * * Constructor takes in colormap * * * //
    Colorbar(Colormap map) {

	_label = ""; _value = 0.0;
	
	// Set the default colormap
	setColormap(map);

	// Set the default data
	setColormapData(0.0, 1.0);	
    }

    // * * * Constructor (default colormap * * * //
    Colorbar() { this(new RainbowColormapFactory().getColormap()); }

    // * * * Set the labels * * * //
    void setLabel(String label) { _label = label; }
    
    // * * * Set colormap * * * //
    void setColormap(Colormap colormap) { _colormap = colormap; }

    // * * * Set the data to colormap * * * //
    void setColormapData(float min, float max){
	_min = min; _max = max;
	
	_colormap.setRange(0.0, 1.0);
	float inc = (1.0)/_numPts;
	_colorsLow = new color[_numPts];
	_colorsHigh = new color[_numPts];

	for(int i = 0; i < _numPts; i++) {
	    _colorsLow[i] = _colormap.getColor((inc*i));
	    _colorsHigh[i] = _colormap.getColor((inc*i));
	}
	_value = _max/2.0;
    }

    // * * * Set the current point * * * //
    void setCurrentPt(float p){ _value = p; }

    // * * * Set the origin * * * ??
    void setOrigin(Point o){ _origin = o; }

    // * * * Draw the colorbar * * * //
    void drawColorbar() {

	// Colorbar space matrix
	pushMatrix();
	{
	    // Draw the rectangle around the bar
	    translate(width-_wide-60, _high/2+30);
	    stroke(135); strokeWeight(3); noFill();
	    rect(0, 0, _wide-2, _high);
	    point(0,0); point(_wide-2,0);
	    point(_wide-2, _high); point(0, _high);
	    
	    // Draw the colors
	    beginShape(QUAD_STRIP);
	    noStroke();
	    for(int i = 0; i < _numPts; i++){
		fill(_colorsHigh[i]); vertex(i*2, 0);
		fill(_colorsLow[i]);  vertex(i*2, _high);
	    }
	    endShape();
	    
	    // Draw the X crosshair
	    double posX = Etc.affine(_min, _value, _max, 0, _wide);
	    if(_max-_min < 0.00001) posX =_wide/2;
	    stroke(135); strokeWeight(0.25); noFill();
	    line((float)posX-2, 0, (float)posX-2, _high);
	    
	    // Label the colorbar
	    fill(0);
	    text(_label, _wide/2-20,       -5);
	    text(  _min,        -20, _high+15);
	    text(  _max,   _wide-20, _high+15);
	    
	    // Label the crosshair
	    fill(178, 13, 13);
	    text(_value, (float)posX-20, _high+15);
	    
	    // Draw any inherited class stuff
	    drawColorbarDef();
	}
	popMatrix();
    }

    // * * * Draw def for inherited classes * * * //
    void drawColorbarDef() {}
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Left, vertical colorbar
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class LeftColorbar extends Colorbar{

    // * * * Constructor takes in colormap * * * //
    LeftColorbar(Colormap map) { super(map); _high = 25; _label=""; }
    
    // * * * Constructor (default colormap * * * //
    LeftColorbar() { this(new RainbowColormapFactory().getColormap()); }

    // * * * Draw the colorbar * * * //
    void drawColorbar() {

	// Colorbar space matrix
	pushMatrix();
	{
	    // Draw the rectangle around the bar
	    translate(width-_wide-100, 55, 0);
	    stroke(135); strokeWeight(3); noFill();
	    rect(0, 0, _wide-2, _high);
	    point(0,0); point(_wide-2,0);
	    point(_wide-2, _high); point(0, _high);
	    
	    // Draw the colors
	    beginShape(QUAD_STRIP);
	    noStroke();
	    for(int i = 0; i < _numPts; i++){
		fill(_colorsHigh[i]); vertex(i*2, 0);
		fill(_colorsLow[i]);  vertex(i*2, _high);
	    }
	    endShape();
	    
	    // Draw the X crosshair
	    double posX = Etc.affine(_min, _value, _max, 0, _wide);
	    if(_max-_min < 0.00001) posX =_wide/2;
	    stroke(135); strokeWeight(0.25); noFill();
	    line((float)posX-2, 0, (float)posX-2, _high);
	    
	    // Label the colorbar
	    fill(0);
	    text(_label, _wide/2-20,       -5);
	    text(  _min,        -20, _high+15);
	    text(  _max,   _wide-20, _high+15);
	    
	    // Label the crosshair
	    fill(178, 13, 13);
	    text(_value, (float)posX-20, _high+15);
	}
	popMatrix();
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Right, vertical colorbar
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class RightColorbar extends Colorbar{

    // * * * Constructor takes in colormap * * * //
    RightColorbar(Colormap map) { super(map); 
	_high = 25; _wide = 50; _numPts = 25;
	_label = ""; setColormapData(0.0, 1.0);
    }
    
    // * * * Constructor (default colormap * * * //
    RightColorbar() { this(new RainbowColormapFactory().getColormap()); }

    // * * * Draw the colorbar * * * //
    void drawColorbar() {
	
	// Colorbar space matrix
	pushMatrix();
	{    
	    translate(width-_high-30, 45);
	    
	    // Draw the rectangle around the bar
	    stroke(135); strokeWeight(3); noFill();
	    rect(0, 2, _high, _wide-1);
	    point(0,2); point(0, _wide-1);
	    point(_high, _wide-1); point(_high, 2);

	    // Draw the colors
	    beginShape(QUAD_STRIP);
	    noStroke();
	    for(int i = 0; i < _numPts; i++){
		fill(_colorsHigh[i]); vertex(0, _wide-(i*2));
		fill(_colorsLow[i]);  vertex(_high, _wide-(i*2));
	    }
	    endShape();

	    // Draw the X crosshair
	    double posX = Etc.affine(_min, _value, _max, 0, _wide);
	    if(_min == _max) posX =_wide/2;
	    stroke(145, 135, 135); strokeWeight(0.25); noFill();
	    line(0, _wide-(float)posX, _high, _wide-(float)posX);
	    
	    // Label the crosshair
	    fill(178, 13, 13);
	    pushMatrix();
	    rotateZ(radians(-90));
	    translate(-_wide, 40);
	    text(_value, (float)posX, 0);
	    popMatrix();

	    // Label the colorbar
	    fill(0);
	    textAlign(CENTER);
	    text(  _max, _high/2,       -6);
	    text(  _min, _high/2, _wide+13);
	    
	    // Draw the label
	    pushMatrix();
	    rotateZ(radians(-90));
	    fill(0);
	    text(_label,-24, -10);
	    popMatrix();
	    textAlign(LEFT);

	    // Draw any inherited class stuff
	    drawColorbarDef();
	}
	popMatrix();
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// A two-dimensional colorbar
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class TwoDimColorbar extends Colorbar{

    float  _min2, _max2; // Min & max for 2nd dimension
    String _label2;      // Label for the 2nd dimension	
    Point _currentPt;

    // * * * Constructor takes in colormap * * * //
    TwoDimColorbar(TwoDimensionalColormap colormap) { super(colormap);	_currentPt = new Point();  }
     
    // * * * Set the labels * * * //
    void setLabel2(String l2) { _label2 = l2; }

    // * * * Set the current point * * * //
    void setCurrentPt(Point p){ _currentPt = p; }
  

    // * * * Set the data to colormap (2D) * * * //
    void setColormapData(float min1, float max1, float min2, float max2) {
	_min = min1; _max = max1; _min2 = min2; _max2 = max2;
	((TwoDimensionalColormap)_colormap).setRange(0.0, 1.0, 0.0, 1.0);
	float inc =1.0/float(_numPts);
	_colorsLow = new color[_numPts];
	_colorsHigh = new color[_numPts];
	for(int i = 0; i < _numPts; i++) {
	    _colorsLow[i] = ((TwoDimensionalColormap)_colormap).getColor(inc*i, 0.0);
	    _colorsHigh[i] = ((TwoDimensionalColormap)_colormap).getColor(inc*i, 1.0);
	}
	_currentPt = new Point(_min, _min2);
    }
   
    // * * * Draw the colorbar * * * //
    void drawColorbarDef() {

	// Draw the crosshair
	double posY = Etc.affine(_min2, _currentPt.y(), _max2, _high, 0);	
	stroke(135); strokeWeight(0.25); noFill();
	line(0, (float)posY, _wide-2, (float)posY);
	
	// Label the colorbar
	fill(0);
	text(_min2, _wide, _high+5);
	text(_max2, _wide,       5);  

	// Draw the name
	pushMatrix();
	rotateZ(radians(-90));
	translate(-_high+7, -15);
	text(_label2, 0, 0);
	popMatrix();
	
	// Label the cross hair	
	fill(178,13,13);
	text(_currentPt.y(), _wide, (float)posY+5); 
    }
    }
